;
; Copyright (c) 2003-2004 Apple Computer, Inc. All rights reserved.
;
; @APPLE_LICENSE_HEADER_START@
; 
; This file contains Original Code and/or Modifications of Original Code
; as defined in and that are subject to the Apple Public Source License
; Version 2.0 (the 'License'). You may not use this file except in
; compliance with the License. Please obtain a copy of the License at
; http://www.opensource.apple.com/apsl/ and read it before using this
; file.
; 
; The Original Code and all software distributed under the License are
; distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
; EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
; INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
; FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
; Please see the License for the specific language governing rights and
; limitations under the License.
; 
; @APPLE_LICENSE_HEADER_END@
;
;	Change History (most recent first):
;    
; $Log: dnssd.def,v $
; Revision 1.2  2004/07/19 07:43:59  shersche
; export TXTRecord APIs
;
; Revision 1.1  2004/06/18 03:55:11  rpantos
; Move DLL up to main level; additional integration from Scott.
;
; Revision 1.2  2004/03/19 10:07:14  bradley
; Export all DNS-SD API symbols from the DLL so they can be used by clients.
;
; Revision 1.1  2004/02/21 04:16:50  bradley
; DLL wrapper for DNS-SD API.
;
;
;

LIBRARY		dnssd

EXPORTS
	DNSServiceRefSockFD
	DNSServiceProcessResult
	DNSServiceRefDeallocate
	DNSServiceEnumerateDomains
	DNSServiceRegister
	DNSServiceAddRecord
	DNSServiceUpdateRecord
	DNSServiceRemoveRecord
	DNSServiceBrowse
	DNSServiceResolve
	DNSServiceConstructFullName
	DNSServiceCreateConnection
	DNSServiceRegisterRecord
	DNSServiceQueryRecord
	DNSServiceReconfirmRecord
	TXTRecordCreate
	TXTRecordDeallocate
	TXTRecordSetValue
	TXTRecordRemoveValue
	TXTRecordContainsKey
	TXTRecordGetCount
	TXTRecordGetLength
	TXTRecordGetBytesPtr
	TXTRecordGetValuePtr
	TXTRecordGetItemAtIndex
