;
; Copyright (c) 2003-2004 Apple Computer, Inc. All rights reserved.
;
; @APPLE_LICENSE_HEADER_START@
; 
; This file contains Original Code and/or Modifications of Original Code
; as defined in and that are subject to the Apple Public Source License
; Version 2.0 (the 'License'). You may not use this file except in
; compliance with the License. Please obtain a copy of the License at
; http://www.opensource.apple.com/apsl/ and read it before using this
; file.
; 
; The Original Code and all software distributed under the License are
; distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
; EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
; INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
; FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
; Please see the License for the specific language governing rights and
; limitations under the License.
; 
; @APPLE_LICENSE_HEADER_END@
;
;	Change History (most recent first):
;    
; $Log: mdnsNSP.def,v $
; Revision 1.3  2005/01/28 23:48:46  shersche
; <rdar://problem/3942551> Export DllRegisterServer, DllUnregisterServer which can be called from the Installer or regsvr32
; Bug #: 3942551
;
; Revision 1.2  2004/07/13 21:24:28  rpantos
; Fix for <rdar://problem/3701120>.
;
; Revision 1.1  2004/06/18 04:13:44  rpantos
; Move up one level.
;
; Revision 1.1  2004/01/30 03:00:33  bradley
; mDNS NameSpace Provider (NSP). Hooks into the Windows name resolution system to 
; perform .local name lookups using Multicast DNS in all Windows apps.
;
;

LIBRARY		mdnsNSP

EXPORTS
	NSPStartup
	NSPCleanup
	DllRegisterServer	PRIVATE
	DllUnregisterServer	PRIVATE
